﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition
Imports System.Net

Public Class Form
   Private moWMSLayer As WmsRasterLayer
   Private moDefaultExtent As RectangleShape
   Private moProj4Converter As ManagedProj4Projection
   Dim miCurSRID As Integer
   Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
      DisplayWMSLayer()
      btn4326.Enabled = False
      btn26915.Enabled = True

      moProj4Converter = New ManagedProj4Projection(ManagedProj4Projection.GetEsriParametersString(4326), ManagedProj4Projection.GetEsriParametersString(26915))
      moProj4Converter.Open()

   End Sub

   Private Sub DisplayWMSLayer()
      Dim staticOverlay As LayerOverlay

      With WinformsMap1
         .BackColor = Color.White
         .MapUnit = GeographyUnit.DecimalDegree
         moDefaultExtent = New RectangleShape(-95.1921, 36.6286, -89.2734, 33.0465)
         .BackgroundOverlay.BackgroundBrush = New GeoSolidBrush(GeoColor.GeographicColors.Sand)
         staticOverlay = New LayerOverlay()
         .Overlays.Add(staticOverlay)
      End With

      moWMSLayer = New WmsRasterLayer(New Uri("http://www.geostor.arkansas.gov/ArcGIS/services/ORTHO2006-WMS/MapServer/WMSServer"))
      With moWMSLayer
         .IsVisible = True
         .UpperThreshold = Double.MaxValue
         .LowerThreshold = 0
         AddHandler CType(moWMSLayer.ImageSource, WmsRasterSource).SendingWebRequest, AddressOf oWMSRasterSource_SendingWebRequest
         .Open()
         For Each layerName As String In .GetServerLayerNames()
            moWMSLayer.ActiveLayerNames.Add(layerName)
         Next
         staticOverlay.Layers.Add("wmsImageLayerArk", moWMSLayer)
         staticOverlay.Layers.MoveToTop("wmsImageLayerArk")
         Debug.WriteLine(.GetRequestUrl(WinformsMap1.CurrentExtent, WinformsMap1.Width, WinformsMap1.Height))
         WinformsMap1.CurrentExtent = .GetBoundingBox
         .DrawingExceptionMode = DrawingExceptionMode.DrawException
         moWMSLayer.Close()
      End With
      WinformsMap1.CurrentExtent = moDefaultExtent
      WinformsMap1.Refresh()
   End Sub

   Private Sub oWMSRasterSource_SendingWebRequest(sender As Object, e As ThinkGeo.MapSuite.Core.SendingWebRequestEventArgs)
      Dim sRequeststring = e.WebRequest.RequestUri.OriginalString
      Dim sOutputFormat As String = CType(sender, WmsRasterSource).OutputFormat

      If miCurSRID <> 4326 AndAlso miCurSRID <> 0 Then

         '***** Reverse the values for MinX and MinY and also for MaxX and MaxY
         sRequeststring = mRepairRequestString(sRequeststring)
         Dim webRequest__1 As WebRequest = WebRequest.Create(sRequeststring)
         e.WebRequest = webRequest__1
      End If


      If moWMSLayer.Crs = "EPSG:26915" Then
         Debug.WriteLine(e.WebRequest.RequestUri.OriginalString)
      End If


   End Sub
   Private Function mRepairRequestString(sRequestString As String) As String
      Dim iStart As Integer = sRequestString.IndexOf("BBOX")
      Dim iLength = sRequestString.IndexOf("CRS") - iStart
      Dim oCurExt As RectangleShape = WinformsMap1.CurrentExtent

      '***** Strip out the string we want to replace
      Dim sStringToReplace As String = sRequestString.Substring(iStart, iLength)

      '***** Construct a correctly formatted sring for the bounding box
      Dim sBBoxString As String = "BBOX=" & oCurExt.LowerLeftPoint.X.ToString & "," &
                                            oCurExt.LowerLeftPoint.Y.ToString & "," &
                                            oCurExt.UpperRightPoint.X.ToString & "," &
                                            oCurExt.UpperRightPoint.Y.ToString & "&"

      '***** Replace the string in the request
      Dim oRepairedString As String = sRequestString.Replace(sStringToReplace, sBBoxString)


      Return oRepairedString

   End Function

   Private Sub btn26915_Click(sender As Object, e As System.EventArgs) Handles btn26915.Click
      '***** Set the map units to meters
      WinformsMap1.MapUnit = GeographyUnit.Meter
      moWMSLayer.Crs = "EPSG:26915"
      miCurSRID = 26915
      Dim oDefaultExtent As New RectangleShape(-95.1921, 36.6286, -89.2734, 33.0465)

      '***** Convert the default extenstion to the correct projection
      oDefaultExtent = moProj4Converter.ConvertToExternalProjection(oDefaultExtent)
      WinformsMap1.CurrentExtent = oDefaultExtent

      WinformsMap1.Refresh()
      btn26915.Enabled = False
      btn4326.Enabled = True
   End Sub

   Private Sub btn4326_Click(sender As Object, e As System.EventArgs) Handles btn4326.Click
      WinformsMap1.MapUnit = GeographyUnit.DecimalDegree
      moWMSLayer.Crs = "EPSG:4326"
      miCurSRID = 4326

      WinformsMap1.CurrentExtent = moDefaultExtent

      WinformsMap1.Refresh()
      btn4326.Enabled = False
      btn26915.Enabled = True
   End Sub
End Class
